@@ -17,6 +17,7 @@ from contextlib import suppress
 from copy import deepcopy
 from logging.config import dictConfig
 from textwrap import dedent
+from typing import Any, Callable, TypeVar, cast
 
 from traitlets.config.configurable import Configurable, SingletonConfigurable
 from traitlets.config.loader import (
@@ -94,8 +95,10 @@ else:
 
 IS_PYTHONW = sys.executable and sys.executable.endswith("pythonw.exe")
 
+T = TypeVar("T", bound=Callable[..., Any])
 
-def catch_config_error(method):
+
+def catch_config_error(method: T) -> T:
     """Method decorator for catching invalid config (Trait/ArgumentErrors) during init.
 
     On a TraitError (generally caused by bad config), this will print the trait's
@@ -113,7 +116,7 @@ def catch_config_error(method):
             app.log.debug("Config at the time: %s", app.config)
             app.exit(1)
 
-    return inner
+    return cast(T, inner)
 
 
 class ApplicationError(Exception):
@@ -146,21 +149,21 @@ class Application(SingletonConfigurable):
 
     # The name of the application, will usually match the name of the command
     # line application
-    name = Unicode("application")
+    name: t.Union[str, Unicode] = Unicode("application")
 
     # The description of the application that is printed at the beginning
     # of the help.
-    description = Unicode("This is an application.")
+    description: t.Union[str, Unicode] = Unicode("This is an application.")
     # default section descriptions
-    option_description = Unicode(option_description)
-    keyvalue_description = Unicode(keyvalue_description)
-    subcommand_description = Unicode(subcommand_description)
+    option_description: t.Union[str, Unicode] = Unicode(option_description)
+    keyvalue_description: t.Union[str, Unicode] = Unicode(keyvalue_description)
+    subcommand_description: t.Union[str, Unicode] = Unicode(subcommand_description)
 
     python_config_loader_class = PyFileConfigLoader
     json_config_loader_class = JSONFileConfigLoader
 
     # The usage and example string that goes at the end of the help string.
-    examples = Unicode()
+    examples: t.Union[str, Unicode] = Unicode()
 
     # A sequence of Configurable subclasses whose config=True attributes will
     # be exposed at the command line.
@@ -187,16 +190,18 @@ class Application(SingletonConfigurable):
                     yield parent
 
     # The version string of this application.
-    version = Unicode("0.0")
+    version: t.Union[str, Unicode] = Unicode("0.0")
 
     # the argv used to initialize the application
-    argv = List()
+    argv: t.Union[t.List[str], List] = List()
 
     # Whether failing to load config files should prevent startup
-    raise_config_file_errors = Bool(TRAITLETS_APPLICATION_RAISE_CONFIG_FILE_ERROR)
+    raise_config_file_errors: t.Union[bool, Bool] = Bool(
+        TRAITLETS_APPLICATION_RAISE_CONFIG_FILE_ERROR
+    )
 
     # The log level for the application
-    log_level = Enum(
+    log_level: t.Union[str, int, Enum] = Enum(
         (0, 10, 20, 30, 40, 50, "DEBUG", "INFO", "WARN", "ERROR", "CRITICAL"),
         default_value=logging.WARN,
         help="Set the log level by value or name.",
@@ -204,11 +209,11 @@ class Application(SingletonConfigurable):
 
     _log_formatter_cls = LevelFormatter
 
-    log_datefmt = Unicode(
+    log_datefmt: t.Union[str, Unicode] = Unicode(
         "%Y-%m-%d %H:%M:%S", help="The date format used by logging formatters for %(asctime)s"
     ).tag(config=True)
 
-    log_format = Unicode(
+    log_format: t.Union[str, Unicode] = Unicode(
         "[%(name)s]%(highlevel)s %(message)s",
         help="The Logging format template",
     ).tag(config=True)
@@ -350,14 +355,20 @@ class Application(SingletonConfigurable):
 
     #: the alias map for configurables
     #: Keys might strings or tuples for additional options; single-letter alias accessed like `-v`.
-    #: Values might be like "Class.trait" strings of two-tuples: (Class.trait, help-text).
-    aliases: t.Dict[str, str] = {"log-level": "Application.log_level"}
+    #: Values might be like "Class.trait" strings of two-tuples: (Class.trait, help-text),
+    #  or just the "Class.trait" string, in which case the help text is inferred from the
+    #  corresponding trait
+    aliases: t.Dict[t.Union[str, t.Tuple[str, ...]], t.Union[str, t.Tuple[str, str]]] = {
+        "log-level": "Application.log_level"
+    }
 
     # flags for loading Configurables or store_const style flags
     # flags are loaded from this dict by '--key' flags
     # this must be a dict of two-tuples, the first element being the Config/dict
     # and the second being the help string for the flag
-    flags: t.Dict[str, t.Any] = {
+    flags: t.Dict[
+        t.Union[str, t.Tuple[str, ...]], t.Tuple[t.Union[t.Dict[str, t.Any], Config], str]
+    ] = {
         "debug": (
             {
                 "Application": {
@@ -389,12 +400,12 @@ class Application(SingletonConfigurable):
     # this must be a dict of two-tuples,
     # the first element being the application class/import string
     # and the second being the help string for the subcommand
-    subcommands = Dict()
+    subcommands: t.Union[t.Dict[str, t.Tuple[t.Any, str]], Dict] = Dict()
     # parse_command_line will initialize a subapp, if requested
     subapp = Instance("traitlets.config.application.Application", allow_none=True)
 
     # extra command-line arguments that don't set config values
-    extra_args = List(Unicode())
+    extra_args: t.Union[t.List[str], List] = List(Unicode())
 
     cli_config = Instance(
         Config,
@@ -409,11 +420,11 @@ class Application(SingletonConfigurable):
 
     _loaded_config_files = List()
 
-    show_config = Bool(
+    show_config: t.Union[bool, Bool] = Bool(
         help="Instead of starting the Application, dump configuration to stdout"
     ).tag(config=True)
 
-    show_config_json = Bool(
+    show_config_json: t.Union[bool, Bool] = Bool(
         help="Instead of starting the Application, dump configuration to stdout (as JSON)"
     ).tag(config=True)
 
@@ -514,6 +525,7 @@ class Application(SingletonConfigurable):
             for c in cls.mro()[:-3]:
                 classdict[c.__name__] = c
 
+        fhelp: t.Optional[str]
         for alias, longname in self.aliases.items():
             try:
                 if isinstance(longname, tuple):
@@ -528,12 +540,13 @@ class Application(SingletonConfigurable):
                 fhelp = cls.class_get_trait_help(trait, helptext=fhelp).splitlines()
 
                 if not isinstance(alias, tuple):
-                    alias = (alias,)  # type:ignore[assignment]
+                    alias = (alias,)
                 alias = sorted(alias, key=len)  # type:ignore[assignment]
                 alias = ", ".join(("--%s" if len(m) > 1 else "-%s") % m for m in alias)
 
                 # reformat first line
-                fhelp[0] = fhelp[0].replace("--" + longname, alias)
+                assert fhelp is not None
+                fhelp[0] = fhelp[0].replace("--" + longname, alias)  # type:ignore
                 yield from fhelp
                 yield indent("Equivalent to: [--%s]" % longname)
             except Exception as ex:
@@ -552,7 +565,7 @@ class Application(SingletonConfigurable):
         for flags, (cfg, fhelp) in self.flags.items():
             try:
                 if not isinstance(flags, tuple):
-                    flags = (flags,)  # type:ignore[assignment]
+                    flags = (flags,)
                 flags = sorted(flags, key=len)  # type:ignore[assignment]
                 flags = ", ".join(("--%s" if len(m) > 1 else "-%s") % m for m in flags)
                 yield flags
@@ -690,7 +703,9 @@ class Application(SingletonConfigurable):
     @catch_config_error
     def initialize_subcommand(self, subc, argv=None):
         """Initialize a subcommand with argv."""
-        subapp, _ = self.subcommands.get(subc)
+        val = self.subcommands.get(subc)
+        assert val is not None
+        subapp, _ = val
 
         if isinstance(subapp, str):
             subapp = import_item(subapp)
@@ -716,7 +731,7 @@ class Application(SingletonConfigurable):
         This prevents issues such as an alias pointing to InteractiveShell,
         but a config file setting the same trait in TerminalInteraciveShell
         getting inappropriate priority over the command-line arg.
-        Also, loaders expect ``(key: longname)`` and not ````key: (longname, help)`` items.
+        Also, loaders expect ``(key: longname)`` and not ``key: (longname, help)`` items.
 
         Only aliases with exactly one descendent in the class list
         will be promoted.
@@ -737,7 +752,7 @@ class Application(SingletonConfigurable):
         for alias, longname in self.aliases.items():
             if isinstance(longname, tuple):
                 longname, _ = longname
-            cls, trait = longname.split(".", 1)  # type:ignore[assignment]
+            cls, trait = longname.split(".", 1)  # type:ignore
             children = mro_tree[cls]  # type:ignore[index]
             if len(children) == 1:
                 # exactly one descendent, promote alias
@@ -752,7 +767,7 @@ class Application(SingletonConfigurable):
         flags = {}
         for key, (flagdict, help) in self.flags.items():
             newflag: t.Dict[t.Any, t.Any] = {}
-            for cls, subdict in flagdict.items():
+            for cls, subdict in flagdict.items():  # type:ignore
                 children = mro_tree[cls]  # type:ignore[index]
                 # exactly one descendent, promote flag section
                 if len(children) == 1:
@@ -764,19 +779,68 @@ class Application(SingletonConfigurable):
                     newflag[cls] = subdict
 
             if not isinstance(key, tuple):
-                key = (key,)  # type:ignore[assignment]
+                key = (key,)
             for k in key:
                 flags[k] = (newflag, help)
         return flags, aliases
 
     def _create_loader(self, argv, aliases, flags, classes):
-        return KVArgParseConfigLoader(argv, aliases, flags, classes=classes, log=self.log)
+        return KVArgParseConfigLoader(
+            argv, aliases, flags, classes=classes, log=self.log, subcommands=self.subcommands
+        )
+
+    @classmethod
+    def _get_sys_argv(cls, check_argcomplete: bool = False) -> t.List[str]:
+        """Get `sys.argv` or equivalent from `argcomplete`
+
+        `argcomplete`'s strategy is to call the python script with no arguments,
+        so ``len(sys.argv) == 1``, and run until the `ArgumentParser` is constructed
+        and determine what completions are available.
+
+        On the other hand, `traitlet`'s subcommand-handling strategy is to check
+        ``sys.argv[1]`` and see if it matches a subcommand, and if so then dynamically
+        load the subcommand app and initialize it with ``sys.argv[1:]``.
+
+        This helper method helps to take the current tokens for `argcomplete` and pass
+        them through as `argv`.
+        """
+        if check_argcomplete and "_ARGCOMPLETE" in os.environ:
+            try:
+                from traitlets.config.argcomplete_config import get_argcomplete_cwords
+
+                cwords = get_argcomplete_cwords()
+                assert cwords is not None
+                return cwords
+            except (ImportError, ModuleNotFoundError):
+                pass
+        return sys.argv
+
+    @classmethod
+    def _handle_argcomplete_for_subcommand(cls):
+        """Helper for `argcomplete` to recognize `traitlets` subcommands
+
+        `argcomplete` does not know that `traitlets` has already consumed subcommands,
+        as it only "sees" the final `argparse.ArgumentParser` that is constructed.
+        (Indeed `KVArgParseConfigLoader` does not get passed subcommands at all currently.)
+        We explicitly manipulate the environment variables used internally by `argcomplete`
+        to get it to skip over the subcommand tokens.
+        """
+        if "_ARGCOMPLETE" not in os.environ:
+            return
+
+        try:
+            from traitlets.config.argcomplete_config import increment_argcomplete_index
+
+            increment_argcomplete_index()
+        except (ImportError, ModuleNotFoundError):
+            pass
 
     @catch_config_error
     def parse_command_line(self, argv=None):
         """Parse the command line arguments."""
         assert not isinstance(argv, str)
-        argv = sys.argv[1:] if argv is None else argv
+        if argv is None:
+            argv = self._get_sys_argv(check_argcomplete=bool(self.subcommands))[1:]
         self.argv = [cast_unicode(arg) for arg in argv]
 
         if argv and argv[0] == "help":
@@ -788,6 +852,7 @@ class Application(SingletonConfigurable):
             subc, subargv = argv[0], argv[1:]
             if re.match(r"^\w(\-?\w)*$", subc) and subc in self.subcommands:
                 # it's a subcommand, and *not* a flag or class parameter
+                self._handle_argcomplete_for_subcommand()
                 return self.initialize_subcommand(subc, subargv)
 
         # Arguments after a '--' argument are for the script IPython may be
@@ -830,12 +895,12 @@ class Application(SingletonConfigurable):
 
         if not isinstance(path, list):
             path = [path]
-        for path in path[::-1]:
+        for current in reversed(path):
             # path list is in descending priority order, so load files backwards:
-            pyloader = cls.python_config_loader_class(basefilename + ".py", path=path, log=log)
+            pyloader = cls.python_config_loader_class(basefilename + ".py", path=current, log=log)
             if log:
-                log.debug("Looking for %s in %s", basefilename, path or os.getcwd())
-            jsonloader = cls.json_config_loader_class(basefilename + ".json", path=path, log=log)
+                log.debug("Looking for %s in %s", basefilename, current or os.getcwd())
+            jsonloader = cls.json_config_loader_class(basefilename + ".json", path=current, log=log)
             loaded: t.List[t.Any] = []
             filenames: t.List[str] = []
             for loader in [pyloader, jsonloader]:
@@ -882,7 +947,7 @@ class Application(SingletonConfigurable):
         """Load config files by filename and path."""
         filename, ext = os.path.splitext(filename)
         new_config = Config()
-        for (config, filename) in self._load_config_files(
+        for (config, fname) in self._load_config_files(
             filename,
             path=path,
             log=self.log,
@@ -890,9 +955,9 @@ class Application(SingletonConfigurable):
         ):
             new_config.merge(config)
             if (
-                filename not in self._loaded_config_files
+                fname not in self._loaded_config_files
             ):  # only add to list of loaded files if not previously loaded
-                self._loaded_config_files.append(filename)
+                self._loaded_config_files.append(fname)
         # add self.cli_config to preserve CLI config priority
         new_config.merge(self.cli_config)
         self.update_config(new_config)
@@ -941,6 +1006,8 @@ class Application(SingletonConfigurable):
         """generate default config file from Configurables"""
         lines = ["# Configuration file for %s." % self.name]
         lines.append("")
+        lines.append("c = get_config()  #" + "noqa")
+        lines.append("")
         classes = self.classes if classes is None else classes
         config_classes = list(self._classes_with_config_traits(classes))
         for cls in config_classes:
